package com.qmth.distributed.print.business.service;

import com.baomidou.mybatisplus.extension.service.IService;
import com.deepoove.poi.data.TableRenderData;
import com.qmth.distributed.print.business.bean.dto.report.CourseTargetWebDto;
import com.qmth.distributed.print.business.bean.dto.report.CourseTargetWordDto;
import com.qmth.distributed.print.business.bean.dto.report.ReportCourseEvaluationResultDto;
import com.qmth.distributed.print.business.bean.dto.report.ReportScoreViewDto;
import com.qmth.distributed.print.business.bean.result.report.PaperStructResult;
import com.qmth.distributed.print.business.entity.TRBasicInfo;
import com.qmth.teachcloud.mark.entity.MarkPaper;

import java.util.List;
import java.util.Map;

/**
 * <p>
 * 报告基本情况表 服务类
 * </p>
 *
 * @author wangliang
 * @since 2024-02-18
 */
public interface TRBasicInfoService extends IService<TRBasicInfo> {

    /**
     * 获取课程目标考核分布-分数图
     *
     * @param trBasicInfo
     * @param markPaper
     * @return
     */
    public ReportScoreViewDto getScoreList(TRBasicInfo trBasicInfo, MarkPaper markPaper);

    /**
     * 获取计算参数
     *
     * @param trBasicInfo
     * @param paperStructResultList
     * @return
     */
    public Map<String, Object> getReportCalculateParams(TRBasicInfo trBasicInfo, List<PaperStructResult> paperStructResultList);

    /**
     * 计算课程考核成绩评价结果
     *
     * @param trBasicInfo
     * @param paramsMap
     * @return
     */
    public ReportCourseEvaluationResultDto reportCalculateCourseEvaluationResult(TRBasicInfo trBasicInfo, Map<String, Object> paramsMap);

    /**
     * 获取报告
     *
     * @param trBasicInfo
     * @param markPaper
     * @param userId
     * @return
     */
    public TRBasicInfo getReportView(TRBasicInfo trBasicInfo, MarkPaper markPaper, Long userId);

    /**
     * 构建word动态表格1
     *
     * @param courseTargetWordDtoList
     * @param header2_1List
     * @param header2_2List
     * @return
     */
    public TableRenderData buildWordTable1(List<CourseTargetWordDto> courseTargetWordDtoList,
                                           List<String> header2_1List,
                                           List<String> header2_2List);

    /**
     * 构建word动态表格2
     *
     * @param courseTargetWordDtoList
     * @param courseTargetWebDtoList
     * @param header2_1List
     * @param header2_2List
     * @return
     */
    public TableRenderData buildWordTable2(List<CourseTargetWordDto> courseTargetWordDtoList,
                                           List<CourseTargetWebDto> courseTargetWebDtoList,
                                           List<String> header2_1List,
                                           List<String> header2_2List);

    /**
     * 构建word动态表格3
     *
     * @param courseTargetWordDtoList
     * @return
     */
    public TableRenderData buildWordTable3(List<CourseTargetWordDto> courseTargetWordDtoList);

    /**
     * 构建word动态表格4
     *
     * @param courseTargetWordDtoList
     * @return
     */
    public TableRenderData buildWordTable4(List<CourseTargetWordDto> courseTargetWordDtoList);
}
